"""Robot based on aiohttp

This file is a part of Bookmarks database and Internet robot.

"""

__author__ = "Oleg Broytman <phd@phdru.name>"
__copyright__ = "Copyright (C) 2024 PhiloSoft Design"
__license__ = "GNU GPL"

__all__ = ['robot_aio']


from urllib.parse import urlsplit
import asyncio
import contextvars
import ssl

from aiohttp_socks import ProxyConnector
from python_socks import parse_proxy_url
from urllib3.util.ssl_ import create_urllib3_context
import aioftp
import aiohttp
import aiohttp.client_exceptions

from Robots.base import robot_base
from Robots.multi_mixin import multi_mixin


current_href = contextvars.ContextVar('current_href')


class robot_aio(multi_mixin, robot_base):
    def __init__(self, *args, **kw):
        robot_base.__init__(self, *args, **kw)
        multi_mixin.__init__(self, *args, **kw)

        # We need one event loop for the entire application
        # so that we can save pending tasks between calls to self.wait().
        # This also means we cannot use asyncio.run().
        self.loop = loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        # Rename self.log, create one log_lines list per URL
        self.file_log = self.log
        del self.log
        self.logs = {}  # Map {href: [log lines]}

    def __getattr__(self, attr):
        if attr != 'log':
            raise AttributeError(attr)
        href = current_href.get()
        return self.logs[href].append

    def version_str(self):
        return 'aiohttp/%s; multi' % aiohttp.__version__

    async def check_bookmark_async_log(self, bookmark):
        current_href.set(bookmark.href)
        await self.check_bookmark_async(bookmark)

    async def get_url(self, url, req_headers):
        if url not in self.logs:
            self.logs[url] = []
        current_href.set(url)
        return await super(robot_aio, self).get_url(url, req_headers)

    async def get(self, url, req_headers, use_proxy=False):
        if url.startswith('ftp://'):
            error, body = await _get_ftp(
                url, timeout=self.ftp_timeout,
            )
            if error is not None:
                error = str(error)
                return error, None, None, None
            return None, None, None, body

        if use_proxy:
            proxy = self.proxy
        else:
            proxy = None

        error, status, resp_headers, body = await _get_http(
            url, req_headers, proxy=proxy,
            timeout=self.timeout,
        )
        return error, status, resp_headers, body

    def wait(self):
        self.loop.run_until_complete(self.wait_async())

    async def wait_async(self):
        bookmarks = self.bookmarks
        pending = self.pending

        free_workers = self.max_urls - len(pending)
        if bookmarks and (free_workers > 0):  # we have job and free workers
            for href in bookmarks:
                bookmark, _, task = bookmarks[href]
                if task is not None:  # it's already pending
                    continue
                task = asyncio.create_task(
                    self.check_bookmark_async_log(bookmark))
                bookmarks[href] = [bookmark, None, task]
                if href not in self.logs:
                    self.logs[href] = []
                pending.add(task)
                task.href = href

                free_workers -= 1
                if free_workers == 0:
                    break

        if pending:
            done, pending = await asyncio.wait(
                pending, timeout=self.timeout+1,
                return_when=asyncio.FIRST_COMPLETED)
        self.pending = pending

        for task in done:
            bookmark, _, old_task = bookmarks.pop(task.href)
            assert old_task is task
            log = self.file_log
            log_lines = self.logs.pop(bookmark.href)
            log('Checked: %s' % bookmark.href)
            if log_lines:
                for line in log_lines:
                    log(line)
            else:
                if hasattr(bookmark, 'error'):
                    log('   Error: %s' % bookmark.error)
                else:
                    log('   No logs')

    def get_ftp_welcome(self):
        return ''  # We don't store welcome message yet

    def stop(self):
        super(robot_aio, self).stop()
        self.loop.close()


async def _get_http(url, req_headers={}, proxy=None, timeout=60):
    connector = None
    if proxy and proxy.startswith('socks5'):
        if proxy.startswith('socks5h://'):
            proxy = proxy.replace('socks5h://', 'socks5://')
            rdns = True
        else:
            rdns = False
        proxy_type, proxy_host, proxy_port, pusername, ppassword = \
            parse_proxy_url(proxy)
        connector = ProxyConnector(
            proxy_type=proxy_type, host=proxy_host, port=proxy_port,
            username=pusername, password=ppassword, rdns=rdns,
        )
        proxy = None

    timeout = aiohttp.ClientTimeout(connect=timeout, total=timeout)
    ssl_context = create_urllib3_context(cert_reqs=0,
                                         ciphers='ALL:@SECLEVEL=1')
    try:
        async with aiohttp.ClientSession(
            connector=connector, timeout=timeout
        ) as session:
            async with session.get(
                    url, headers=req_headers,
                    proxy=proxy, allow_redirects=False,
                    ssl_context=ssl_context) as resp:
                return None, resp.status, resp.headers, await resp.read()
    except (
        ConnectionResetError,
        asyncio.TimeoutError, ssl.SSLError,
        aiohttp.client_exceptions.ClientError
    ) as e:
        return e, None, None, None


async def _get_ftp(url, timeout=60):
    split_results = urlsplit(url)
    path = split_results.path or '/'
    user = split_results.username or 'anonymous'
    password = split_results.password or 'ftp'
    host = split_results.hostname
    port = split_results.port or 21

    lines = []
    try:
        async with aioftp.Client.context(
            host, port, user=user, password=password,
            socket_timeout=timeout, path_timeout=timeout,
        ) as client:
            async for _path, _info in client.list(path):
                lines.append('%s %s' % (_info, _path))
    except (OSError, aioftp.errors.AIOFTPException) as e:
        return e, None
    return None, '\n'.join(lines)
