"""HTML Parser using lxml.html

This file is a part of Bookmarks database and Internet robot.

"""

__author__ = "Oleg Broytman <phd@phdru.name>"
__copyright__ = "Copyright (C) 2010-2024 PhiloSoft Design"
__license__ = "GNU GPL"

__all__ = ['parse_html']


import re
from lxml.etree import ParserError
from lxml.html import fromstring
from .bkmk_ph_util import HTMLParser


def parse_html(html_text, charset=None, log=None):
    if not html_text:
        return None
    try:
        html_tree = fromstring(html_text)
    except ValueError as e:
        if e.args[0].startswith(
            'Unicode strings with encoding declaration are not supported.'
            ' Please use bytes input'
        ):
            if not charset:
                match = re.search(
                    '<\\?xml version="(\\d|.)+" encoding="([^"]+)"\\?>',
                    html_text, re.U)
                if match:
                    charset = match.group(2)
            if charset:
                html_text = html_text.encode(charset)
                html_tree = fromstring(html_text)
            else:
                return None
        else:
            raise
    except ParserError:
        return None

    title = html_tree.findtext('head/title')
    if title is None:
        title = html_tree.findtext('title')

    meta = html_tree.findall('head/meta')
    for m in meta:
        if m.get('http-equiv', '').lower() == 'content-type':
            meta_content = m.get("content")
            if meta_content:
                try:
                    meta_charset = \
                        meta_content.lower().split('charset=')[1].split(';')[0]
                    break
                except IndexError:
                    meta_charset = False
        elif m.get('charset', ''):
            meta_charset = m.get('charset').lower()
            break
    else:
        meta_charset = False

    for m in meta:
        if m.get('http-equiv', '').lower() == 'refresh':
            refresh = m.get("content")
            break
    else:
        refresh = None

    for link in html_tree.findall('head/link'):
        if link.get('rel', '').lower() in ('icon', 'shortcut icon'):
            icon = link.get("href")
            break
    else:
        icon = None

    if (title is None) and (refresh is None) and (icon is None):
        return None
    return HTMLParser(charset, meta_charset, title, refresh, icon)
